#ifndef wasix_function_pointer_h
#define wasix_function_pointer_h
// Dynamic types used by the wasix runtime types APIs

#include <wasi/api_wasi.h>
#include <wasi/api_wasix.h>

#ifdef __cplusplus
extern "C"
{
#endif

#define wasix_function_pointer_t __wasi_function_pointer_t

#ifdef __cplusplus
}
#endif
#endif